/*
 *  MBtypes.h
 *  MacBill
 *
 *  Created by Ishioka Hiroshi on Thu Dec 20 2001.
 *  Copyright (c) 2001-2002 Ishioka Hiroshi. All rights reserved.
 *
 */

typedef struct MBPicture MBPicture;
typedef struct MBMCursor MBMCursor;
